package org.rexxla.bsf.engines.rexx;

import java.util.*;

/** Class modelled after the <a href="http://www.ooRexx.org">ooRexx</a> class &quot;Supplier&quot;. Needs the {@link ArrayWrapper} class.
 *  Allows iterating over an array (regardless of its dimension). In the case an entry of the array is
 *  <code>null</code>, that entry is not reflected in the supplier object.
 *
 * @author Rony G. Flatscher
 * @version 1.1.1, 2005-06-02
 * @since   2003-02-08 by rgf
 */
public class Supplier implements Enumeration
{

    /** Private array storing the values of the array. */
    private Object values[]=null;

    /** Private array storing the indices of the array in form of Strings. */
    private Object indices[]=null;

    /** Stores the index of the index/value pair pointing at. */
    private int index=0;

    /** Stores the length of the arrays. */
    private int length=0;

    /** Tests whether an index/value pair is available.
     *
     * @return <code>true</code>, if an index/value pair is available, <code>false</code> else.
     */
    public boolean available()
    {
        return (length>0 && index<length);  //
    }

    /** Move to the next item/value pair, if any.
     */
    public void next()
    {
        index++;            // position on next item/value pair
    }

    /** Returns the item (value) part of the supplier object.
     *
     * @return the item (value) part of the supplier object
     */
    public Object item()
    {
        return values[index];   // return the item (value)
    }

    /** Returns the index part of the supplier object.
     *
     * @return the index part of the supplier object
     */
    public Object index()
    {
        return indices[index];  // return the index
    }


    /** Creates an instance of the class using the {@link ArrayWrapper} class.
     *
     * @param arrObject an array (of any dimension)
     * @param bRexxStyle <code>true</code> format index-part in Object Rexx style (indexing starts with
     *         <code>1</code> and multiple indices are delimited with commas), <code>false</code> format index-part in
     *         Java style (indexing starts with <code>0</code> and multiple indices are delimited by
     *         enclosing each index in square brackets)
     */
    public Supplier (Object arrObject, boolean bRexxStyle)
    {
        // ArrayWrapper aw = ArrayWrapper.getArrayWrapper(arrObject);  // wrap up the array
        ArrayWrapper aw = new ArrayWrapper(arrObject);  // wrap up the array
        Object objArr [] = aw.makearray(true, bRexxStyle); // create the two arrays
        values  = (Object []) objArr[0];
        indices = (Object []) objArr[1];
        length=indices.length;
    }

    /** Creates an instance of the class using the {@link ArrayWrapper} class. Uses the Java style
     *  of indexing for the index part of the supplier object.
     *
     * @param arrObject an array (of any dimension)
     */
    public Supplier (Object arrObject)
    {
        this(arrObject, false);             // default: use Java style indexing (starts with 0)
    }


    /** Creates an instance of the class.
     *
     * @param values a single-dimensioned array containing the values (items)
     * @param indices a single-dimensioned array containing the indices of <code>v</code>
     */
    public Supplier (Object [] values, Object [] indices)
    {
        this.values  = values;              // save values (items) array
        this.indices = indices;             // save indices array
        length  = this.indices.length;      // save length (number of entries) of array
    }



    // Enumeration interface implementation
    /** Implements the interface {@link java.util.Enumeration#hasMoreElements() java.util.Enumeration.hasMoreElements()}.
     */
    public boolean hasMoreElements()
    {
        return this.available();
    }

    /** Implements the interface {@link java.util.Enumeration#nextElement() java.util.Enumeration.nextElement()}.
     *
     * @exception NoSuchElementException raised, if using the Enumerator interface and moving
     *                                   beyond the end of the supplier object.
     */
    public Object nextElement() throws NoSuchElementException
    {
        this.next();            // move to next index/element pair
        if (index>=length)
        {
            throw new NoSuchElementException( this+": exceeding total number of ["+length+"] elements!");
        }
        return this.item();     // return element
    }
}

